@@ -1,10 +1,10 @@
 """Distributed Task Queue"""
 
-VERSION = (1, 0, 6)
+VERSION = (2, 0, 0)
 
 __version__ = ".".join(map(str, VERSION[0:3])) + "".join(VERSION[3:])
 __author__ = "Ask Solem"
-__contact__ = "askh@opera.com"
+__contact__ = "ask@celeryproject.org"
 __homepage__ = "http://github.com/ask/celery/"
 __docformat__ = "restructuredtext"
 
