@@ -1,9 +1,48 @@
-from celery.utils.functional import curry
-import operator
+import inspect
+import sys
+import types
+
+from copy import deepcopy
+
+import pickle as pypickle
 try:
-    import cPickle as pickle
+    import cPickle as cpickle
 except ImportError:
-    import pickle
+    cpickle = None
+
+if sys.version_info < (2, 6):
+    # cPickle is broken in Python <= 2.5.
+    # It unsafely and incorrectly uses relative instead of absolute imports,
+    # so e.g.:
+    #       exceptions.KeyError
+    # becomes:
+    #       celery.exceptions.KeyError
+    #
+    # Your best choice is to upgrade to Python 2.6,
+    # as while the pure pickle version has worse performance,
+    # it is the only safe option for older Python versions.
+    pickle = pypickle
+else:
+    pickle = cpickle or pypickle
+
+
+# BaseException was introduced in Python 2.5.
+try:
+    _error_bases = (BaseException, )
+except NameError:
+    _error_bases = (SystemExit, KeyboardInterrupt)
+
+unwanted_base_classes = (StandardError, Exception) + _error_bases + (object, )
+
+
+if sys.version_info < (2, 5):
+
+    # Prior to Python 2.5, Exception was an old-style class
+    def subclass_exception(name, parent, unused):
+        return types.ClassType(name, (parent,), {})
+else:
+    def subclass_exception(name, parent, module):
+        return type(name, (parent,), {'__module__': module})
 
 
 def find_nearest_pickleable_exception(exc):
@@ -21,16 +60,23 @@ def find_nearest_pickleable_exception(exc):
     :rtype: :exc:`Exception`
 
     """
+    cls = exc.__class__
+    getmro_ = getattr(cls, "mro", None)
 
-    unwanted = (Exception, BaseException, object)
-    is_unwanted = lambda exc: any(map(curry(operator.is_, exc), unwanted))
+    # old-style classes doesn't have mro()
+    if not getmro_:
+        # all Py2.4 exceptions has a baseclass.
+        if not getattr(cls, "__bases__", ()):
+            return
+        # Use inspect.getmro() to traverse bases instead.
+        getmro_ = lambda: inspect.getmro(cls)
 
-    mro_ = getattr(exc.__class__, "mro", lambda: [])
-    for supercls in mro_():
-        if is_unwanted(supercls):
+
+    for supercls in getmro_():
+        if supercls in unwanted_base_classes:
             # only BaseException and object, from here on down,
             # we don't care about these.
-            return None
+            return
         try:
             exc_args = getattr(exc, "args", [])
             superexc = supercls(*exc_args)
@@ -39,14 +85,14 @@ def find_nearest_pickleable_exception(exc):
             pass
         else:
             return superexc
-    return None
+    return
 
 
 def create_exception_cls(name, module, parent=None):
     """Dynamically create an exception class."""
     if not parent:
         parent = Exception
-    return type(name, (parent, ), {"__module__": module})
+    return subclass_exception(name, parent, module)
 
 
 class UnpickleableExceptionWrapper(Exception):
@@ -86,7 +132,17 @@ class UnpickleableExceptionWrapper(Exception):
         self.exc_module = exc_module
         self.exc_cls_name = exc_cls_name
         self.exc_args = exc_args
-        super(Exception, self).__init__(exc_module, exc_cls_name, exc_args)
+        Exception.__init__(self, exc_module, exc_cls_name, exc_args)
+
+    @classmethod
+    def from_exception(cls, exc):
+        return cls(exc.__class__.__module__,
+                   exc.__class__.__name__,
+                   getattr(exc, "args", []))
+
+    def restore(self):
+        return create_exception_cls(self.exc_cls_name,
+                                    self.exc_module)(*self.exc_args)
 
 
 def get_pickleable_exception(exc):
@@ -96,13 +152,9 @@ def get_pickleable_exception(exc):
         return nearest
 
     try:
-        pickle.dumps(exc)
-    except pickle.PickleError:
-        excwrapper = UnpickleableExceptionWrapper(
-                        exc.__class__.__module__,
-                        exc.__class__.__name__,
-                        getattr(exc, "args", []))
-        return excwrapper
+        pickle.dumps(deepcopy(exc))
+    except Exception:
+        return UnpickleableExceptionWrapper.from_exception(exc)
     return exc
 
 
@@ -110,7 +162,5 @@ def get_pickled_exception(exc):
     """Get original exception from exception pickled using
     :meth:`get_pickleable_exception`."""
     if isinstance(exc, UnpickleableExceptionWrapper):
-        exc_cls = create_exception_cls(exc.exc_cls_name,
-                                       exc.exc_module)
-        return exc_cls(*exc.exc_args)
+        return exc.restore()
     return exc
