@@ -353,3 +353,17 @@ except ImportError:
                 yield tup
         except IndexError:
             pass
+
+############## itertools.chain.from_iterable ################################
+from itertools import chain
+
+
+def _compat_chain_from_iterable(iterables):
+    for it in iterables:
+        for element in it:
+            yield element
+
+#try:
+#    chain_from_iterable = getattr(chain, "from_iterable")
+#except AttributeError:
+chain_from_iterable = _compat_chain_from_iterable
