@@ -1,7 +1,9 @@
 from datetime import datetime
 
 from celery import conf
+from celery.backends import default_backend
 from celery.registry import tasks
+from celery.utils import timeutils
 from celery.worker.revoke import revoked
 from celery.worker.control.registry import Panel
 
@@ -9,11 +11,40 @@ TASK_INFO_FIELDS = ("exchange", "routing_key", "rate_limit")
 
 
 @Panel.register
-def revoke(panel, task_id, **kwargs):
+def revoke(panel, task_id, task_name=None, **kwargs):
     """Revoke task by task id."""
     revoked.add(task_id)
+    backend = default_backend
+    if task_name: # Use custom task backend (if any)
+        try:
+            backend = tasks[task_name].backend
+        except KeyError:
+            pass
+    backend.mark_as_revoked(task_id)
     panel.logger.warn("Task %s revoked" % (task_id, ))
-    return True
+    return {"ok": "task %s revoked" % (task_id, )}
+
+
+@Panel.register
+def enable_events(panel):
+    dispatcher = panel.listener.event_dispatcher
+    if not dispatcher.enabled:
+        dispatcher.enable()
+        dispatcher.send("worker-online")
+        panel.logger.warn("Events enabled by remote.")
+        return {"ok": "events enabled"}
+    return {"ok": "events already enabled"}
+
+
+@Panel.register
+def disable_events(panel):
+    dispatcher = panel.listener.event_dispatcher
+    if dispatcher.enabled:
+        dispatcher.send("worker-offline")
+        dispatcher.disable()
+        panel.logger.warn("Events disabled by remote.")
+        return {"ok": "events disabled"}
+    return {"ok": "events already disabled"}
 
 
 @Panel.register
@@ -26,6 +57,12 @@ def rate_limit(panel, task_name, rate_limit, **kwargs):
     :param rate_limit: New rate limit.
 
     """
+
+    try:
+        timeutils.rate(rate_limit)
+    except ValueError, exc:
+        return {"error": "Invalid rate limit string: %s" % exc}
+
     try:
         tasks[task_name].rate_limit = rate_limit
     except KeyError:
@@ -107,4 +144,4 @@ def ping(panel, **kwargs):
 @Panel.register
 def shutdown(panel, **kwargs):
     panel.logger.critical("Got shutdown from remote.")
-    raise SystemExit()
+    raise SystemExit
